@@ -1,4 +1,4 @@
-""" 
+"""
 pyweblib.forms - class library for handling <FORM> input
 (c) by Michael Stroeder <michael@stroeder.com>
 
@@ -11,10 +11,10 @@ This module only works with Python 2.0 since all string parameters
 are assumed to be Unicode objects and string methods are used instead
 string module.
 
-$Id: forms.py,v 1.36 2006/03/10 15:43:15 michael Exp $
-""" 
+$Id: forms.py,v 1.47 2012/03/16 13:09:48 michael Exp $
+"""
 
-__version__ = '0.15.0'
+__version__ = '0.16.0'
 
 import sys,types,re,urllib
 import helper
@@ -27,6 +27,7 @@ def escapeHTML(s):
   s = s.replace('&','&#38;')
   s = s.replace('<','&#60;')
   s = s.replace('>','&#62;')
+  s = s.replace("'",'&#39;')
   s = s.replace('"','&#34;')
   s = s.replace(':','&#58;')
   s = s.replace('=','&#61;')
@@ -34,13 +35,14 @@ def escapeHTML(s):
   s = s.replace('}','&#125;')
   s = s.replace('(','&#40;')
   s = s.replace(')','&#41;')
+  s = s.replace('`','&#96;')
   return s
 
 
 class Field:
   """
   Base class for all kind of single input fields.
-  
+
   In most cases this class is not used directly
   since derivate classes for most types of input fields exist.
   """
@@ -107,11 +109,11 @@ class Field:
       return self.value
     else:
       return self.value[0]
-    
+
   def setDefault(self,default):
     """
     Set the default of a field.
-    
+
     Mainly this is used by the application if self.default shall
     be changed after initializing the field object.
     """
@@ -132,14 +134,14 @@ class Field:
       return pattern
     else:
       return pattern, 0
-    
+
   def setRegex(self,pattern):
     """
     Set the regex pattern for validating this field.
-    
+
     Mainly this is used if self._re shall be changed after
     the field object was initialized.
-    
+
     pattern
         Either a string containing a regex pattern,
         a tuple (pattern string, pattern options) or None.
@@ -163,7 +165,7 @@ class Field:
   def _validateFormat(self,value):
     """
     Check format of the user's value for this field.
-    
+
     Empty input (zero-length string) are valid in any case.
     You might override this method to change this behaviour.
     """
@@ -266,10 +268,10 @@ class Textarea(Field):
       '<textarea %stitle="%s" name="%s" %s rows="%d" cols="%d">%s</textarea>' % (
         self.idAttrStr(id_value),
         self.titleHTML(title),
-	self.name,
-	self._accessKeyAttr(),
-	self.rows,self.cols,
-	self._defaultHTML(default)
+        self.name,
+        self._accessKeyAttr(),
+        self.rows,self.cols,
+        self._defaultHTML(default)
       )
     )
 
@@ -301,8 +303,8 @@ class Input(Field):
 
   def inputHTML(self,default=None,id_value=None,title=None):
     return self.inputHTMLTemplate % (
-	'<input %stitle="%s" name="%s" %s maxlength="%d" size="%d" value="%s">' % (
-	self.idAttrStr(id_value),
+      '<input %stitle="%s" name="%s" %s maxlength="%d" size="%d" value="%s">' % (
+        self.idAttrStr(id_value),
         self.titleHTML(title),
         self.name,
         self._accessKeyAttr(),
@@ -333,13 +335,13 @@ class HiddenInput(Input):
     else:
       default_str = ''
     return self.inputHTMLTemplate % (
-	'<input type="hidden" %stitle="%s" name="%s" %s  value="%s">%s' % (
-	  self.idAttrStr(id_value),
-          self.titleHTML(title),
-          self.name,
-          self._accessKeyAttr(),
-          default_html,
-          default_str
+      '<input type="hidden" %stitle="%s" name="%s" %s  value="%s">%s' % (
+        self.idAttrStr(id_value),
+        self.titleHTML(title),
+        self.name,
+        self._accessKeyAttr(),
+        default_html,
+        default_str
       )
     )
 
@@ -363,15 +365,12 @@ class File(Input):
 
   def inputHTML(self,default=None,id_value=None,title=None,mimeType=None):
     return self.inputHTMLTemplate % (
-	'<input type="file" %stitle="%s" name="%s" %s maxlength="%d" size="%d" accept="%s" value="%s">' % (
-	self.idAttrStr(id_value),
+      '<input type="file" %stitle="%s" name="%s" %s accept="%s">' % (
+        self.idAttrStr(id_value),
         self.titleHTML(title),
         self.name,
         self._accessKeyAttr(),
-        self.maxLen,
-        self.size,
-        mimeType or self.mimeType,
-        self._defaultHTML(default)
+        mimeType or self.mimeType
       )
     )
 
@@ -385,7 +384,7 @@ class Password(Input):
   def inputHTML(self,default=None,id_value=None,title=None):
     return self.inputHTMLTemplate % (
       '<input %stitle="%s" name="%s" %s maxlength="%d" size="%d" type="password" value="%s">' % (
-	self.idAttrStr(id_value),
+        self.idAttrStr(id_value),
         self.titleHTML(title),
         self.name,
         self._accessKeyAttr(),
@@ -423,31 +422,39 @@ class Radio(Field):
       [('value1','description1),('value2','description2),..]
       for options with different option value and description.
     """
-    self.options = options or []
-    Field.__init__(
-      self,name,text,0,maxValues,'',required,default,accessKey
-    )
     self.setOptions(options)
     self.setDefault(default)
+    Field.__init__(
+      self,name,text,self.maxLen,maxValues,'',required,default,accessKey
+    )
+
+  def _validateFormat(self,value):
+    """
+    Check format of the user's value for this field.
+
+    Empty input (zero-length string) are valid in any case.
+    You might override this method to change this behaviour.
+    """
+    if value and (not value in self.optionValues):
+      raise InvalidValueFormat(
+        self.name,
+        self.text.encode(self.charset),
+        value.encode(self.charset)
+      )
 
   def setOptions(self,options):
-    maxLen = 0
+    self.optionValues = {}
+    self.maxLen = 0
     if options:
       optionValues = []
       for i in options:
         if type(i) is types.TupleType:
-          optionValue,optionText=i
-	else:
-          optionValue=i
-        optionValues.append(optionValue)
-        if len(optionValue)>maxLen:
-	  maxLen=len(optionValue)
-      pattern   = u'(%s)' % u'|'.join(map(re.escape,optionValues))
-    else:
-      pattern   = u''
-    self.maxLen = maxLen
+          optionValue = i[0]
+        else:
+          optionValue = i
+        self.optionValues[optionValue] = None
+      self.maxLen = max(map(len,self.optionValues.keys()))
     self.options  = options
-    self.setRegex(pattern)
 
   def inputHTML(self,default=None,id_value=None,title=None):
     s = []
@@ -460,13 +467,14 @@ class Radio(Field):
       s.append("""
         <input
           type="radio"
-          id="%s"
+          %s
           title="%s"
           name="%s"
           %s
           value="%s"
           %s
-        >%s<br>""" % (
+        >%s<br>
+        """ % (
           self.idAttrStr(id_value),
           self.titleHTML(title),
           self.name.encode(self.charset),
@@ -474,14 +482,14 @@ class Radio(Field):
           optionValue,
           ' checked'*(optionValue==default_value),
           optionText
-	)
+        )
       )
     return self.inputHTMLTemplate % '\n'.join(s)
 
   def setDefault(self,default):
     """
     Set the default of a default field.
-    
+
     Mainly this is used if self.default shall be changed after
     initializing the field object.
     """
@@ -542,19 +550,29 @@ class Select(Radio):
     default_value = self._defaultValue(default)
     for i in self.options:
       if type(i) is types.TupleType:
-        optionValue,optionText = i
+        try:
+          optionValue,optionText,optionTitle = i
+        except ValueError:
+          optionTitle = None
+          optionValue,optionText = i
       else:
+        optionTitle = None
         optionValue = optionText = i
       if self.multiSelect:
-	option_selected = optionValue in default_value
+        option_selected = optionValue in default_value
       else:
-	option_selected = (optionValue==default_value) or (self.ignoreCase and optionValue.lower()==default_value.lower())
+        option_selected = (optionValue==default_value) or (self.ignoreCase and optionValue.lower()==default_value.lower())
+      if optionTitle:
+        optionTitle_attr = u' title="%s"' % escapeHTML(optionTitle.encode(self.charset))
+      else:
+        optionTitle_attr = ''
       s.append(
-	'<option value="%s"%s>%s</option>' % (
-	  escapeHTML(optionValue.encode(self.charset)),
-	  ' selected'*(option_selected),
-	  escapeHTML(optionText.encode(self.charset)),
-	)
+        '<option value="%s"%s%s>%s</option>' % (
+          escapeHTML(optionValue.encode(self.charset)),
+          optionTitle_attr,
+          ' selected'*(option_selected),
+          escapeHTML(optionText.encode(self.charset)),
+        )
       )
     s.append('</select>')
     return self.inputHTMLTemplate % '\n'.join(s)
@@ -588,8 +606,8 @@ class Checkbox(Field):
         self.idAttrStr(id_value),
         self.titleHTML(title),
         self.name,
-	self._accessKeyAttr(),
-	self._defaultValue(default),' checked'*(checked)
+        self._accessKeyAttr(),
+        self._defaultValue(default),' checked'*(checked)
       )
     )
 
@@ -631,7 +649,7 @@ class Keygen(Field):
 class FormException(Exception):
   """
   Base exception class to indicate form processing errors.
-  
+
   Attributes:
   args          unstructured List of parameters
   """
@@ -645,7 +663,7 @@ class FormException(Exception):
 class InvalidRequestMethod(FormException):
   """
   Exception raised when HTTP request method was invalid.
-  
+
   Attributes:
   method        string containing method used
   """
@@ -657,7 +675,7 @@ class InvalidRequestMethod(FormException):
 class InvalidFormEncoding(FormException):
   """
   The form data is malformed.
-  
+
   Attributes:
   param         name/value causing the exception
   """
@@ -844,15 +862,16 @@ class Form:
     else:
       return default
 
-  def hiddenInputFields(self,outf=sys.stdout,ignoreFieldNames=[]):
+  def hiddenInputFields(self,outf=sys.stdout,ignoreFieldNames=None):
     """
     Output all parameters as hidden fields.
-    
+
     outf
         File object for output.
     ignoreFieldNames
         Names of parameters to be excluded.
     """
+    ignoreFieldNames=ignoreFieldNames or []
     for f in [
       self.field[p]
       for p in self.declaredFieldNames
@@ -867,7 +886,7 @@ class Form:
     return # Form.hiddenInputFields()
 
   def _parseFormUrlEncoded(self,maxContentLength,ignoreEmptyFields,ignoreUndeclaredFields,stripValues,unquote):
-      
+
     if self.request_method=='POST':
       query_string = self.inf.read(int(self.env['CONTENT_LENGTH']))
     elif self.request_method=='GET':
@@ -889,17 +908,17 @@ class Form:
           name,value = param.split('=',1)
         except ValueError:
           raise InvalidFormEncoding(param)
-	name = unquote(name).strip()
-        
+        name = unquote(name).strip()
+
         if not name in self.declaredFieldNames:
           if ignoreUndeclaredFields:
             continue
           else:
             raise UndeclaredFieldName(name)
-        
-	value = unquote(value)
+
+        value = unquote(value)
         if stripValues:
-  	  value = value.strip()
+          value = value.strip()
 
         contentLength += len(value)
         # Gesamtlaenge der Daten noch zulaessig?
@@ -909,12 +928,12 @@ class Form:
         f = self.field[name]
 
         # input is empty string?
-	if value or (not ignoreEmptyFields):
-	  # Input is stored in field instance
+        if value or (not ignoreEmptyFields):
+          # Input is stored in field instance
           f.setValue(value)
           # Add name of field to list of input keys
           if not name in self.inputFieldNames:
-	    self.inputFieldNames.append(name)
+            self.inputFieldNames.append(name)
 
     return #_parseFormUrlEncoded()
 
@@ -938,7 +957,7 @@ class Form:
       for value in parts[name]:
 
 #        if stripValues:
-#  	  value = value.strip()
+#     value = value.strip()
 
         contentLength += len(value)
         # Gesamtlaenge der Daten noch zulaessig?
@@ -949,11 +968,11 @@ class Form:
 
         # input is empty string?
         if value or (not ignoreEmptyFields):
-	  # Input is stored in field instance
+          # Input is stored in field instance
           f.setValue(value)
           # Add name of field to list of input keys
           if not name in self.inputFieldNames:
-	    self.inputFieldNames.append(name)
+            self.inputFieldNames.append(name)
 
     return # _parseMultipartFormData()
 
@@ -1002,7 +1021,7 @@ class Form:
     missing_params = []
     for n,f in self.field.items():
       if f.required and not (f.name in self.inputFieldNames):
-	missing_params.append((f.name,f.text))
+        missing_params.append((f.name,f.text))
     if missing_params:
       raise ParamsMissing(missing_params)
 
