@@ -6,41 +6,20 @@ This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
 
-$Id: sslenv.py,v 1.6 2003/01/25 23:56:49 michael Exp $
+$Id: sslenv.py,v 1.11 2011/04/16 13:57:25 michael Exp $
 """
 
-__version__ = '0.6.1'
+__version__ = '0.6.3'
+
+from forms import escapeHTML
 
 import sys,os,re,string
 
-def t612html4(s):
-  """
-  Convert T.61 character representation to HTML-4 character representation
-  """
-  new = ''
-  slashpos = string.find(s,'\\x')
-  while slashpos!=-1:
-    if (s[slashpos]==0) or (s[slashpos]>0 and s[slashpos-1]!='\\'):
-      new = new+s[0:slashpos]+'&#%d;' % string.atoi(s[slashpos+2:slashpos+4],16)
-      s = s[slashpos+4:]
-    else:
-      new = new+s[0:slashpos-1]
-      s = s[slashpos+1:]
-    slashpos = string.find(s,'\\x')
-  return new+s
-
-def asn12html4(s):
-  """
-  Convert BMPString to HTML-4 character representation
-  """
-  return t612html4(string.replace(s,'\\x00',''))
-
-
 def GetAllSSLEnviron(env=None):
   """
   Get all SSL-related environment vars and return mod_ssl
   compatible dictionary.
-  
+
   mod_ssl compatible names are preferred. ApacheSSL names
   are used as fallback.
   """
@@ -250,12 +229,12 @@ def GetAllSSLEnviron(env=None):
 def SecLevel(env,acceptedciphers,valid_dn_regex='',valid_idn_regex=''):
   """
   Determine Security Level of SSL session.
-  
+
   Returns:
-  0	no SSL at all
-  1	SSL-connection and cipher used is in acceptedciphers
-  2	like 1 but client also has sent client certificate
-  	matching valid_dn_regex and valid_idn_regex.
+  0  no SSL at all
+  1  SSL-connection and cipher used is in acceptedciphers
+  2  like 1 but client also has sent client certificate
+     matching valid_dn_regex and valid_idn_regex.
   """
   https_env = GetAllSSLEnviron(env)
   if https_env and https_env.get('SSL_CIPHER','') in acceptedciphers:
@@ -265,9 +244,9 @@ def SecLevel(env,acceptedciphers,valid_dn_regex='',valid_idn_regex=''):
       dn_rm = re.compile(valid_dn_regex).match(ssl_client_s_dn)
       idn_rm = re.compile(valid_idn_regex).match(ssl_client_i_dn)
       if (dn_rm) and (idn_rm):
-	return 2
-      else:  
-	return 1
+        return 2
+      else:
+        return 1
     else:
       return 1
   else:
@@ -280,86 +259,69 @@ def PrintSecInfo(env,acceptedciphers,valid_dn_regex='',valid_idn_regex='',f=sys.
   """
   seclevel = SecLevel(env,acceptedciphers,valid_dn_regex,valid_idn_regex)
   https_env = GetAllSSLEnviron(env)
-  f.write("""<h3>Security level</h3><p>Current security level is: <strong>%d</strong></p>
-           <table cellspacing=5%%>
-	   <tr>
-	     <td align=center width=10%%>0</td>
-	     <td>no encryption at all</td>
-	   </tr>
-	   <tr>
-	     <td align=center>1</td>
-	     <td>Session is encrypted with SSL and cipher is accepted</td>
-	   </tr>
-	   <tr>
-	     <td align=center>2</td>
-	     <td>Client presented valid certificate,<br>
-	     the DN of the certified object matches "<CODE>%s</CODE>"<br>
-	     and the DN of the certifier matches "<CODE>%s</CODE>"</td>
-	   </tr>
-	   </table>
-	   """ % (seclevel,valid_dn_regex,valid_idn_regex))
+  f.write("""<h3>Security level</h3>
+<p>Current security level is: <strong>%d</strong></p>
+<table cellspacing="5%%" summary="Possible SSL/TLS security levels">
+<tr>
+  <td align=center width=10%%>0</td>
+  <td>no encryption at all</td>
+</tr>
+<tr>
+  <td align=center>1</td>
+  <td>Session is encrypted with SSL and cipher is accepted</td>
+</tr>
+<tr>
+  <td align=center>2</td>
+  <td>
+    Client presented valid certificate,
+    the DN of the certified object matches &quot;<code>%s</code>&quot;
+    and the DN of the certifier matches &quot;<code>%s</code>&quot;
+  </td>
+</tr>
+</table>
+   """ % (seclevel,valid_dn_regex,valid_idn_regex))
 
   if seclevel>=1:
+    SSL_PROTOCOL = https_env.get('SSL_PROTOCOL')
     SSL_CIPHER_ALGKEYSIZE = https_env.get('SSL_CIPHER_ALGKEYSIZE')
     SSL_CIPHER = https_env.get('SSL_CIPHER')
     SSL_CIPHER_USEKEYSIZE = https_env.get('SSL_CIPHER_USEKEYSIZE')
     SSL_SERVER_S_DN = https_env.get('SSL_SERVER_S_DN')
     SSL_SERVER_I_DN = https_env.get('SSL_SERVER_I_DN')
 
-    f.write("""You connected with cipher <strong>%s</strong>, key size <strong>%s Bit</strong>, actually used key size <strong>%s Bit</strong>.<p>
+    f.write("""<p><strong>%s</strong> connection with cipher <strong>%s</strong>,
+key size <strong>%s Bit</strong>, actually used key size <strong>%s Bit</strong>.</p>
 <h3>Server certificate</h3>
-<table summary="Server certificate">
-  <tr>
-    <td>
-      <dl>
-	<dt>This certificate belongs to:</dt>
-	<dd>%s</dd>
-      </dl>
-    </td>
-    <td>
-      <dl>
-	<dt>This certificate was issued by:</dt>
-	<dd>%s</dd>
-      </dl>
-    </td>
-  </tr>
-</table>
+<dl>
+  <dt>Subject-DN:</dt>
+  <dd>%s</dd>
+  <dt>Issuer-DN:</dt>
+  <dd>%s</dd>
+</dl>
 """ % (
+  SSL_PROTOCOL,
   SSL_CIPHER,
   SSL_CIPHER_ALGKEYSIZE,
   SSL_CIPHER_USEKEYSIZE,
-  string.join(string.split(asn12html4(SSL_SERVER_S_DN),'/'),'<br>'),
-  string.join(string.split(asn12html4(SSL_SERVER_I_DN),'/'),'<br>')
+  escapeHTML(SSL_SERVER_S_DN),
+  escapeHTML(SSL_SERVER_I_DN),
 ))
 
   if seclevel>=2:
 
-    SSL_CLIENT_I_DN = https_env.get('SSL_CLIENT_I_DN',
-			 https_env.get('SSL_CLIENT_IDN',
-			 ''))
-    SSL_CLIENT_S_DN = https_env.get('SSL_CLIENT_S_DN',
-		      https_env.get('SSL_CLIENT_DN',
-		      ''))
+    SSL_CLIENT_I_DN = https_env.get('SSL_CLIENT_I_DN',https_env.get('SSL_CLIENT_IDN','')
+    )
+    SSL_CLIENT_S_DN = https_env.get('SSL_CLIENT_S_DN',https_env.get('SSL_CLIENT_DN',''))
 
     f.write("""<h3>Your client certificate</h3>
-<table summary="Client certificate">
-  <tr>
-    <td>
-      <dl>
-	<dt>This certificate belongs to:</dt>
-	<dd>%s</dd>
-      </dl>
-    </td>
-    <td>
-      <dl>
-	<dt>This certificate was issued by:</dt>
-	<dd>%s</dd>
-      </dl>
-    </td>
-  </tr>
-</table>
+<dl>
+  <dt>Subject-DN:</dt>
+  <dd>%s</dd>
+  <dt>Issuer-DN:</dt>
+  <dd>%s</dd>
+</dl>
 """ % (
-  string.join(string.split(asn12html4(SSL_CLIENT_S_DN),'/'),'<br>'),
-  string.join(string.split(asn12html4(SSL_CLIENT_I_DN),'/'),'<br>')
+  escapeHTML(SSL_CLIENT_S_DN),
+  escapeHTML(SSL_CLIENT_I_DN),
 ))
 
